*****************************************************************
*								*
*	 MORROW DESIGNS CP/M VERS 2.2 COLD BOOT LOADER.		*
*		  CBIOS REVISION E.2,  3/4/82.			*
*								*
*        SQUEEZED DOWN TO DJ2D/B ONLY 12 MAR 85 RHP		*
*								*
*	     8 INCH FLOPPY DISK BOOT LOADER FOR THE		*
*            MORROW DESIGNS DISK JOCKEY 2D/B (DJ2DB)		*
*								*
* THE 'ORDER' COLUMN IS THE INTERLEAVE SEQUENCE USED BY THE	*
* LOADER DURING THE LOAD.					*
*								*
*   TRACK   SECTOR    SYSGEN	LOAD ORDER	NAME		*
*								*
*	0	 1	 900	FF00	 0	BOOT LOADER	*
*	0	 2	 980			UNUSED		*
*	0	 3	 A00					*
*	0	 4	 A80					*
*	0	 5	 B00	9100	 1	CCP		*
*	0	 6	 B80	9180	12			*
*	0	 7	 C00	9200	 2			*
*	0	 8	 C80	9280	13			*
*	0	 9	 D00	9300	 3			*
*	0	10	 D80	9380	14			*
*	0	11	 E00	9400	 4			*
*	0	12	 E80	9480	15			*
*	0	13	 F00	9500	 5			*
*	0	14	 F80	9580	16			*
*	0	15	1000	9600	 6			*
*	0	16	1080	9680	17			*
*	0	17	1100	9700	 7			*
*	0	18	1180	9780	18			*
*	0	19	1200	9800	 8			*
*	0	20	1280	9880	19			*
*	0	21	1300	9900	 9	BDOS		*
*	0	22	1380	9980	20			*
*	0	23	1400	9A00	10			*
*	0	24	1480	9A80	21			*
*	0	25	1500	9B00	11			*
*	0	26	1580	9B80	22			*
*								*
* TRACK 1 IS RECORDED IN DOUBLE DENSITY FORMAT.  THERE ARE	*
* 1024 BYTES PER SECTOR.					*
*								*
*	1	 1	1600	9C00	 4			*
*	1	 2	1A00	A000	 1			*
*	1	 3	1E00	A400	 5	CBIOS (@ A700H)	*
*	1	 4	2200	A800	 2			*
*	1	 5	2600	AC00	 6			*
*	1	 6	2A00	B000	 3			*
*	1	 7	2E00	B400	 7			*
*	1	 8	3200	B800		UNUSED		*
*								*
* THREE SPARE SECTORS (TRACK 0, SECTORS 2 TO 4) HAVE BEEN	*
* PROVIDED FOR A MORE ADVANCED BOOT LOADER AT A LATER DATE.	*
*								*
* THE WARM BOOT LOADER STARTS ON TRACK 0, SECTOR 5 AND		*
* CONTINUES THROUGH TO TRACK 1 SECTOR 3.  ONLY THE FIRST	*
* 3/4 K BYTES OF TRACK 1, SECTOR 3 IS LOADED SINCE CP/M		*
* REQUIRES THAT THE WARM BOOT LOADER LOAD UP TO THE START	*
* OF (BUT NOT PAST) THE CBIOS JUMP TABLE.			*
*								*
*****************************************************************

*****************************************************************
*								*
* THE FOLLOWING TABLE GIVES A GENERAL IDEA AS TO WHERE THE	*
* VARIOUS PARTS OF OF THE OPERATING SYSTEM ARE IN MEMORY.	*
*								*
* THE ONLY CHANGES TO THE MAP THAT I SEE IN THE FUTURE IS	*
* THE INCREASING THE SPACE FOR THE UNINITIALIZED TABLES		*
* FOLLOWING THE CBIOS.  THE AMOUNT OF CODE AND TABLE SPACE	*
* THAT CAN ACTUALLY BE LOADED FROM THE DISK IS FIXED BY THE	*
* AMOUNT OF SPACE AVAILABLE ON THE SYSTEM TRACKS.		*
*								*
*								*
* SINCE 512 BYTES ARE RESERVED FOR THE COLD BOOT LOADER WE	*
* HAVE A TOTAL OF 9.5K BYTES FOR THE OPERATING SYSTEM.  OUT	*
* OF THIS 5.5K BYTES ARE USED BY THE (CCP + BDOS) LEAVING	*
* US WITH A TOTAL OF 4K BYTES OF LOADED CODE AND DATA SPACE	*
* TO PLAY WITH.  RIGHT NOW WE ARE USING ALL OF THIS SPACE	*
* SO ANY MAJOR ADDITIONS WILL HAVE TO RESULT IN A LITTLE	*
* (LOT?) OF CODE SHUFFELING OR IN THE CREATION OF A CBIOS	*
* THAT SIMPLY WILL NOT FIT ON A SMALL DISK DRIVE.		*
*								*
*	SYSGEN	48K	56K	60K	62K	64K		*
*	 IMAGE	CP/M	CP/M	CP/M	CP/M	CP/M		*
*								*
*	 900	~~~~	~~~~	~~~~	~~~~	~~~~	LOADER	*
*	 B00	9500	B500	C500	CD00	D500	CCP	*
*	1300	9D00	BD00	CD00	D500	DD00	BDOS	*
*	2100	AB00	CB00	DB00	E300	ED00	CBIOS	*
*	3100	BB00	DB00	EB00	F300	FB00	TABLES	*
*	35FF	BFFF	DFFF	EFFF	F7FF	FFFF	THE END	*
*								*
*	~~~~	8D00	AD00	BD00	C500	CD00	DDT	*
*								*
*****************************************************************
	
MSIZE	EQU	56		;MEMORY SIZE OF TARGET CP/M
			
BIOSLN	EQU	10H		;BIOS LENGTH
CODLEN	EQU	11H		;CODE LENGTH
	
CCPLN	EQU	800H
BDOSLN	EQU	0E00H

SIZE	EQU	(MSIZE*1024)	
CCP	EQU	SIZE-(BIOSLN*100H+CCPLN+BDOSLN)
BDOS	EQU	CCP+CCPLN
BIOS	EQU	CCP+CCPLN+BDOSLN

CBOOT	EQU	BIOS		;COLD BOOT ADDRESS FOR CP/M
LOADDR	EQU	CCP		;LOAD ADDRESS FOR FLOPPY

RETRIES	EQU	10		;MAXIMUM # OF DISK RETRIES

*****************************************************************
*								*
* THE FOLLOWING EQUATES ARE FOR THE DISK JOCKEY 2D/B.		*	
*								*
*****************************************************************

ORIGIN	EQU	0E000H		;ORGIN OF DJ 2D MOD B PROM
DJRAM	EQU	ORIGIN+400H
				;DISK JOCKEY 2D MOD B ROUTINES
TKZERO	EQU	ORIGIN+9H	;TRACK 0 SEEK
TRKSET	EQU	ORIGIN+0CH	;SET TRACK
SETSEC	EQU	ORIGIN+0FH	;SET SECTOR
SETDMA	EQU	ORIGIN+12H	;SET DMA ADDRESS
DREAD	EQU	ORIGIN+15H	;READ SECTOR
DMAST	EQU	ORIGIN+24H	;GET DMA ADDRESS
STATUS	EQU	ORIGIN+27H	;DISK STATUS
DSKERR	EQU	ORIGIN+2AH	;FLASH ERROR LIGHT
SETDEN	EQU	ORIGIN+2DH	;SET DENSITY

BOOT	EQU	DJRAM+300H	;UPPER 3/4 OF ON BOARD FLOPPY RAM
OFFSET	EQU	900H-BOOT	;DDT OFFSET

	ORG	BOOT

*****************************************************************
*								*
* COLD BOOT LOADER FOR THE DISK JOCKEY 2D REVISION B CONTROLLER	*
*								*
*****************************************************************

T0BOOT	MVI	A,5-2		;FIRST SECTOR - 2
NEWSEC	EQU	$-1
	INR	A		;UPDATE SECTOR #
	INR	A
	CPI	27		;SIZE OF TRACK IN SECTORS + 1
TRKSIZ	EQU	$-1
	JC	NOWRAP		;SKIP IF NOT AT END OF TRACK
	JNZ	T1BOOT		;DONE WITH THIS TRACK
EXIT	EQU	$-2
	SUI	27-6		;BACK UP TO SECTOR 6
BACKUP	EQU	$-1
	LXI	H,LOADDR-80H	;MEMORY ADDRESS OF SECTOR - 100H
NXTDMA	EQU	$-2
	SHLD	NEWDMA
NOWRAP	STA	NEWSEC		;SAVE THE UPDATED SECTOR #
	MOV	C,A
	CALL	SETSEC		;SET UP THE SECTOR
	LXI	H,LOADDR-100H	;MEMORY ADDRESS OF SECTOR - 100H
NEWDMA	EQU	$-2
	LXI	D,100H		;UPDATE DMA ADDRESS
SECSIZ	EQU	$-2
	DAD	D
NOWRP	SHLD	NEWDMA		;SAVE THE UPDATED DMA ADDRESS
	MOV	B,H
	MOV	C,L
	CALL	SETDMA		;SET UP THE NEW DMA ADDRESS
	LXI	B,RETRIES*100H+0;MAXIMUM # OF ERRORS, TRACK #
NXTRTY	EQU	$-2
FREAD	PUSH	B
	CALL	TRKSET		;SET UP THE PROPER TRACK
	CALL	DREAD		;READ THE SECTOR
	POP	B
	JNC	T0BOOT		;CONTINUE IF NO ERROR
	DCR	B
	JNZ	FREAD		;KEEP TRYING IF ERROR
	JMP	DSKERR		;TOO MANY ERRORS, FLASH THE LIGHT

T1BOOT	LXI	H,CBOOT		;WE JUMP TO CBOOT NEXT TIME
	SHLD	EXIT
	MVI	C,1		;SELECT DOUBLE DENSITY
	CALL	SETDEN
	XRA	A		;FIRST SECTOR - 2
	STA	NEWSEC
	MVI	A,8		;SIZE OF (LOGICAL) TRACK + 1
	STA	TRKSIZ
	DCR	A		;NUMBER OF SECTORS TO BACK UP
	STA	BACKUP
	LXI	H,LOADDR+0700H	;DMA START ADDRESS FOR FIRST REVOLUTION - 2048
	SHLD	NEWDMA
	LXI	H,LOADDR+0300H	;DMA START ADDRESS FOR SECOND REVOLUTION - 2048
	SHLD	NXTDMA
	LXI	H,2048		;DIFFERENCE BETWEEN DMA ADDRESSES
	SHLD	SECSIZ
	LXI	H,RETRIES*100H+1;MAXIMUM # OF ERRORS, TRACK #
	SHLD	NXTRTY
	JMP	T0BOOT		;GO LOAD IN TRACK 1

	END
 F00	9900	 4			*
                *	0	 5	1100	9B00	 5			*
                *	0	 6	1300	9D00	 6	BDOS		*
                *	0	 7	1500	9F00	 7			*
                *	0	 8	1700	A100	 8			*
                *	0	 9	1900	A300	 9			*
                *	0	10	1B00	A500	10			*
                *	0	11	1D00	A700	11			*
                *	0	12	1F00	A900	12			*
                *	0	13	2100	AB00	13	CBIOS		*
                *	0	14	2300	AD00	14			*
                *	0	15	2500	AF00	15			*
                *	0	16	2700	B100	16			*
                *	0	17	2900	B300	17			*
                *	0	18	2B00	B500	18			*
                *	0	19	2D00	B700	19			*
                *	0	20	2F00	B900	20			*
                *	0	21	3100			UNUSED		*
                *								*
                * THE WARM BOOT LOAD SEQUENCE STARTS AT TRACK 0, SECTOR 2	*
                * AND GOES STRAIGHT THROUGH TO SECTOR 12.  THERE IS STILL	*
                * PLENTY OF ROOM LEFT IN THIS LOADER FOR MORE ADVANCED		*
                * THINGS LIKE SECTOR INTERLEAVING ALTHOUGH THIS IS HARDLY	*
                * NECESSARY ON A HARD DISK.					*
                *								*
                *								*
                *	SHUGART SA1000 DISK INTERFACE BOOT LOADER FOR THE	*
                *	 MORROW DESIGNS HARD DISK DMA CONTROLLER (HDDMA)	*
                *								*
                *   TRACK   SECTOR    SYSGEN	LOAD ORDER	NAME		*
                *	0	 1	 900	 100	 0	COLD BOOT + CCP	*
                *	0	 2	 D00	9300	 1			*
                *	0	 3	1100	9700	 2			*
                *	0	 4	1500	9B00	 3	BDOS (@ 9D00)	*
                *	0	 5	1900	9F00	 4			*
                *	0	 6	1D00	A300	 5			*
                *	0	 7	2100	A700	 6	CBIOS		*
                *	0	 8	2500	AB00	 7			*
                *	0	 9	2900	AF00	 8			*
                *								*
                *	1	10	2D00	B300	 9			*
                *								*
                * SINCE 1K BYTE SECTORS WERE IMPLEMENTED ON THIS DISK;  	*
                * TRACK 0, SECTOR 1 CONTAINS BOTH THE COLD BOOT LOADER AND	*
                * PART OF THE CCP.  THE COLD BOOT LOADER RELOCATES THIS		*
                * PEICE OF THE CCP TO IT PROPER RESTING PLACE AS PART OF	*
                * THE BOOT PROCESS.						*
                *								*
                *****************************************************************
                
                *****************************************************************
                *								*
                * THE FOLLOWING TABLE GIVES A GENERAL IDEA AS TO WHERE THE	*
                * VARIOUS PARTS OF OF THE OPERATING SYSTEM ARE IN MEMORY.	*
                *								*
                * THE ONLY CHANGES TO THE MAP THAT I SEE IN THE FUTURE IS	*
                * THE INCREASING THE SPACE FOR THE UNINITIALIZED TABLES		*
                * FOLLOWING THE CBIOS.  THE AMOUNT OF CODE AND TABLE SPACE	*
                * THAT CAN ACTUALLY BE LOADED FROM THE DISK IS FIXED BY THE	*
                * AMOUNT OF SPACE AVAILABLE ON THE SYSTEM TRACKS.		*
                *								*
                * OUR MOST RESTRICTIVE (SMALLEST) DRIVE IS THE 5 1/4 INCH	*
                * 'MINNIE FLOPPY'.  THIS DRIVE HAS 20 512 BYTE SECTORS FOR	*
                * A TOTAL OF 10K BYTES ON THE SYSTEM TRACKS.  THE 8 INCH	*
                * FLOPPY DISK DRIVE IS ALSO VERY CLOSE TO BEING FILLED UP.	*
                *								*
                * SINCE 512 BYTES ARE RESERVED FOR THE COLD BOOT LOADER WE	*
                * HAVE A TOTAL OF 9.5K BYTES FOR THE OPERATING SYSTEM.  OUT	*
                * OF THIS 5.5K BYTES ARE USED BY THE (CCP + BDOS) LEAVING	*
                * US WITH A TOTAL OF 4K BYTES OF LOADED CODE AND DATA SPACE	*
                * TO PLAY WITH.  RIGHT NOW WE ARE USING ALL OF THIS SPACE	*
                * SO ANY MAJOR ADDITIONS WILL HAVE TO RESULT IN A LITTLE	*
                * (LOT?) OF CODE SHUFFELING OR IN THE CREATION OF A CBIOS	*
                * THAT SIMPLY WILL NOT FIT ON A SMALL DISK DRIVE.		*
                *								*
                *	SYSGEN	48K	56K	60K	62K	64K		*
                *	 IMAGE	CP/M	CP/M	CP/M	CP/M	CP/M		*
                *								*
                *	 900	~~~~	~~~~	~~~~	~~~~	~~~~	LOADER	*
                *	 B00	9500	B500	C500	CD00	D500	CCP	*
                *	1300	9D00	BD00	CD00	D500	DD00	BDOS	*
                *	2100	AB00	CB00	DB00	E300	ED00	CBIOS	*
                *	3100	BB00	DB00	EB00	F300	FB00	TABLES	*
                *	35FF	BFFF	DFFF	EFFF	F7FF	FFFF	THE END	*
                *								*
                *	~~~~	8D00	AD00	BD00	C500	CD00	DDT	*
                *								*
                *****************************************************************
                	
 0000 =         MICRON	EQU	0		;SET TO 1 FOR MICRONIX BOOT LOADER
                
                	IF	MICRON EQ 0
                
 0038 =         MSIZE	EQU	56		;MEMORY SIZE OF TARGET CP/M
                			
 0010 =         BIOSLN	EQU	10H		;BIOS LENGTH
 0011 =         CODLEN	EQU	11H		;CODE LENGTH
                	
 0800 =         CCPLN	EQU	800H
 0E00 =         BDOSLN	EQU	0E00H
                
 E000 =         SIZE	EQU	(MSIZE*1024)	
 BA00 =         CCP	EQU	SIZE-(BIOSLN*100H+CCPLN+BDOSLN)
 C200 =         BDOS	EQU	CCP+CCPLN
 D000 =         BIOS	EQU	CCP+CCPLN+BDOSLN
                
 D000 =         CBOOT	EQU	BIOS		;COLD BOOT ADDRESS FOR CP/M
 BA00 =         LOADDR	EQU	CCP		;LOAD ADDRESS FOR FLOPPY
                
                	ELSE			;MICRONIX BOOT LOADER
                
                CBOOT	EQU	0100H		;COLD BOOT ADDRESS FOR THE LOADER
                LOADDR	EQU	0100H
                	
                				;IF THE LOAD ADDRESS IS MOVED FORWARD FROM
                				;0100 THEN THE STARTING EXTENDED ADDRESS FOR
                				;THE DJDMA BOOT LOADER SHOULD BE ADJUSTED.
                	ENDIF
                
 000A =         RETRIES	EQU	10		;MAXIMUM # OF DISK RETRIES
                
                *****************************************************************
                *								*
                * ONLY ONE OF THE FOLLOWING EQUATES SHOULD BE SET.  THE		*
                * OTHERS SOULD BE 0.  THESE EQUATES DEFINE THE BOOT LOADER	*
                * THAT IS TO BE USED.						*
                *								*
                ******************************************************************
                
 0000 =         MAXHD	EQU	0		;SET TO BOOT AN HDC3 CONTROLLER
 0000 =         MAXMW	EQU	0		;SET TO BOOT A HDDMA CONTROLLER
 0001 =         MAXFD	EQU	1		;SET TO BOOT A DJ2D/B CONTROLLER
 0000 =         MAXDM	EQU	0		;SET TO BOOT A DJDMA CONTROLLER WITH 8 INCH
 0000 =         MAXMF	EQU	0		;SET TO BOOT A DJDMA CONTROLLER WITH 5 1/4 INCH
                
                
                *****************************************************************
                *								*
                * THE FOLLOWING EQUATES ARE FOR THE HARD DISK CONTROLLER 3.	*
                *								*
                *****************************************************************
                
                	IF	MAXHD NE 0
                HDORG	EQU	50H		;HARD DISK CONTROLLER
                HDSTAT	EQU	HDORG		;HARD DISK STATUS
                HDCNTL	EQU	HDORG		;HARD DISK CONTROL
                HDDATA	EQU	HDORG+3		;HARD DISK DATA
                HDFUNC	EQU	HDORG+2		;HARD DISK FUNCTION
                HDCMND	EQU	HDORG+1		;HARD DISK COMMAND
                HDRESLT	EQU	HDORG+1		;HARD DISK RESULT
                RETRY	EQU	2		;RETRY BIT OF RESULT
                TKZ	EQU	1		;TRACK ZERO BIT OF STATUS
                OPDONE	EQU	2		;OPERACTION DONE BIT OF STATUS
                COMPLT	EQU	4		;COMPLETE BIT OF STATUS
                TMOUT	EQU	8		;TIME OUT BIT OF STATUS
                WFAULT	EQU	10H		;WRITE FAULT BIT OF STATUS
                DRVRDY	EQU	20H		;DRIVE READY BIT OF STATUS
                INDX	EQU	40H		;INDEX BIT OF STATUS
                PSTEP	EQU	4		;STEP BIT OF FUNCTION
                NSTEP	EQU	0FBH		;STEP BIT MASK OF FUNCTION
                HDRLEN	EQU	4		;SECTOR HEADER LENGTH
                SECLN	EQU	512		;SECTOR DATA LENGTH
                WENABL	EQU	0FH		;WRITE ENABLE
                WRESET	EQU	0BH		;WRITE RESET OF FUNCTION
                SCENBL	EQU	5		;CONTROLLER CONTROL
                DSKCLK	EQU	7		;DISK CLOCK FOR CONTROL
                MDIR	EQU	0F7H		;DIRECTION MASK FOR FUNCTION
                NULL	EQU	0FCH		;NULL COMMAND
                IDBUFF	EQU	0		;INITIALIZE DATA COMMAND
                ISBUFF	EQU	8		;INITIALIZE HEADER COMMAND
                RSECT	EQU	1		;READ SECTOR COMMAND
                WSECT	EQU	5		;WRITE SECTOR COMMAND
                	ENDIF
                
                *****************************************************************
                *								*
                * THE FOLLOWING EQUATES ARE FOR THE HARD DISK DMA.		*
                *								*
                *****************************************************************
                	
                	IF	MAXMW NE 0
                CYL	EQU	153		;SPECIFICATIONS FOR A SEAGATE TECHNOLOGY 506
                HEADS	EQU	4		;NUMBER OF HEADS PER CYLINDER
                SPT	EQU	9		;SECTORS PER TRACK
                PRECOMP	EQU	64		;CYLINDER TO START WRITE PRECOMENSATION
                LOWCURR	EQU	128		;CYLINDER TO START LOW CURRENT
                STEPDLY	EQU	30		;STEP DELAY (0-12.7 MILLISECONDS)
                HEADDLY	EQU	20		;SETTLE DELAY (0-255 MILLISECONDS)
                SECTSIZ	EQU	7		;SECTOR SIZE CODE (MUST BE 3 FOR THIS CBIOS)
                				; 0 =  128 BYTE SECTORS
                				; 1 =  256 BYTE SECTORS
                				; 3 =  512 BYTE SECTORS
                				; 7 = 1024 BYTE SECTORS (DEFAULT FOR CP/M)
                				; F = 2048 BYTE SECTORS
                
                				;DEFINE CONTROLLER COMMANDS
                DMAREAD	EQU	0		;READ SECTOR
                DMAWRIT	EQU	1		;WRITE SECTOR
                DMARHED	EQU	2		;FIND A SECTOR
                DMAWHED	EQU	3		;WRITE HEADERS (FORMAT A TRACK)
                DMALCON	EQU	4		;LOAD DISK PARAMETERS
                DMASSTA	EQU	5		;SENSE DISK DRIVE STATUS
                DMANOOP	EQU	6		;NULL CONTROLLER OPERATION
                
                RESET	EQU	54H		;RESET CONTROLLER
                ATTN	EQU	55H		;SEND A CONTROLLER ATTENTION
                
                CHAN	EQU	50H		;DEFAULT CHANNEL ADDRESS
                STEPOUT	EQU	10H		;STEP DIRECTION OUT
                STEPIN	EQU	0		;STEP DIRECTION IN
                BAND1	EQU	40H		;NO PRECOMP, HIGH CURRENT
                BAND2	EQU	0C0H		;PRECOMP, HIGH CURRENT
                BAND3	EQU	80H		;PRECOMP, LOW CURRENT
                TRACK0	EQU	1		;TRACK ZERO STATUS
                WFAULT	EQU	2		;WRITE FAULT FROM DRIVE
                DREADY	EQU	4		;DRIVE READY
                	ENDIF
                
                *****************************************************************
                *								*
                * THE FOLLOWING EQUATES ARE FOR THE DISK JOCKEY 2D/B.		*	
                *								*
                *****************************************************************
                
                	IF	MAXFD NE 0
 F800 =         ORIGIN	EQU	0F800H		;ORGIN OF DJ 2D MOD B PROM
 FC00 =         DJRAM	EQU	ORIGIN+400H
                				;DISK JOCKEY 2D MOD B ROUTINES
 F809 =         TKZERO	EQU	ORIGIN+9H	;TRACK 0 SEEK
 F80C =         TRKSET	EQU	ORIGIN+0CH	;SET TRACK
 F80F =         SETSEC	EQU	ORIGIN+0FH	;SET SECTOR
 F812 =         SETDMA	EQU	ORIGIN+12H	;SET DMA ADDRESS
 F815 =         DREAD	EQU	ORIGIN+15H	;READ SECTOR
 F824 =         DMAST	EQU	ORIGIN+24H	;GET DMA ADDRESS
 F827 =         STATUS	EQU	ORIGIN+27H	;DISK STATUS
 F82A =         DSKERR	EQU	ORIGIN+2AH	;FLASH ERROR LIGHT
 F82D =         SETDEN	EQU	ORIGIN+2DH	;SET DENSITY
                	ENDIF
                
                *****************************************************************
                *								*
                * THE FOLLOWING EQUATES ARE FOR THE DISK JOCKEY DMA IF WANTED.	*
                *								*
                *****************************************************************
                
                	IF	(MAXDM NE 0) OR (MAXMF NE 0)
                
                DJKICK	EQU	0EFH		;KICK PORT FOR DJDMA CONTROLLER
                CHANNL	EQU	50H
                	
                	IF	MAXDM NE 0	;8 INCH BOOT LOADER
                TRKOFF	EQU	22*128		;NUMBER OF BYTES LOADED FROM TRACK 0
                	ELSE			;5 1/4 INCH BOOT LOADER
                TRKOFF	EQU	9*512		;NUMBER OF BYTES LOADED FROM TRACK 0
                	ENDIF
                
                SETDMA	EQU	23H		;SET DMA ADDRESS
                DJHALT	EQU	25H		;HALT CONTROLLER
                DJBRAN	EQU	26H		;BRANCH CONTROLLER COMMAND
                REDTRK	EQU	29H		;READ TRACK COMMAND
                
                	ENDIF
                
                *****************************************************************
                *								*
                * DEFINE THE ORIGIN ADDRESS FOR THE VARIOUS BOOT LOADERS.	*
                *								*
                *****************************************************************
                
                	IF	MAXHD NE 0	;HDC3
                BOOT	EQU	BIOS+(BIOSLN*100H)-512	;VERY LAST PART OF CP/M SYSTEM
                	ENDIF
                
                	IF	MAXMW NE 0	;HDDMA
                BOOT	EQU	100H
                	ENDIF
                
                	IF	MAXFD NE 0	;DJ2D/B
 FF00 =         BOOT	EQU	DJRAM+300H	;UPPER 3/4 OF ON BOARD FLOPPY RAM
                	ENDIF
                
                	IF	(MAXDM NE 0) OR (MAXMF NE 0)	;DJDMA
                BOOT	EQU	80H
                	ENDIF
                
 0A00 =         OFFSET	EQU	900H-BOOT	;DDT OFFSET
                
 FF00           	ORG	BOOT
                
                *****************************************************************
                *								*
                * COLD BOOT LOADER FOR A HARD DISK.				*
                *								*
                *****************************************************************
                
                	IF	(MAXHD NE 0) OR (MAXMW NE 0)
                
                	LXI	SP,CSTKHD	;SET UP STACK AT END OF THIS SECTOR
                
                	IF	MAXHD NE 0
                	LXI	B,19*100H+2	;B = SECTOR COUNT, C = SECTOR #
                	CALL	CLODHD
                	JMP	CBOOT		;GO TO CP/M
                	ELSE
                	LXI	H,BOOT+200H	;COPY PART OF CCP UP
                	LXI	D,LOADDR
                	LXI	B,200H
                MOVLOP:	MOV	A,M		;GET A BYTE
                	STAX	D		;SAVE IT
                	INX	H		;BUMP POINTERS
                	INX	D
                	DCX	B		;BUMP COUNTER
                	MOV	A,B		;TEST FOR END
                	ORA	C
                	JNZ	MOVLOP
                	LXI	B,10*100H+2	;B = SECTOR COUNT, C = SECTOR #
                	CALL	CLODHD
                	JMP	CBOOT		;GO TO CP/M
                	ENDIF
                
                CLODHD	PUSH	B		;SAVE SECTOR AND COUNT
                	MOV	A,C		;LOAD SECTOR
                	STA	HDSEC
                	LXI	H,LOADDR-200H	;GET DMA ADDRESS (SELF MODIFYING)
                CDMAHD	EQU	$-2		;STORAGE FOR PREVIOUS DMA ADDRESS
                	IF	MAXHD NE 0
                	LXI	D,200H		;OFFSET TO NEW DMA ADDRESS
                	ELSE
                	LXI	D,400H
                	ENDIF
                	DAD	D		;ADD IN OFFSET, HL = NEW DMA ADDRESS
                	SHLD	CDMAHD		;SAVE NEW DMA ADDRESS
                	CALL	CRDHD		;ATTEMPT A READ
                	POP	B		;RECOVER SECTOR NUMBER AND COUNT
                				;	B = COUNT, C = NUMBER
                	DCR	B		;UPDATE SECTOR COUNT
                	RZ			;ALL DONE ?
                	INR	C		;BUMP SECTOR NUMBER
                	JMP	CLODHD		;CONTINUE READING
                
                *****************************************************************
                *								*
                * CRDHD DOES THE ACTUAL READ FROM THE CONTROLLER, THE DMA	*
                * ADDRESS AND SECTOR # HAVE ALREADY BEEN SET UP.		*
                *								*
                *****************************************************************
                
                	IF	MAXHD NE 0	;LOW LEVEL HDC3 DRIVERS
                CRDHD	LXI	B,RETRIES*100H+1 ;MAXIMUM # OF ATTEMPTS
                CRHD	PUSH	B		;SAVE ERROR COUNT
                	CALL	HDREAD		;ATTEMPT THE READ
                	POP	B		;RESTORE THE ERROR COUNT
                	RNC			;RETURN IF NO ERROR
                	DCR	B		;UPDATE ERROR COUNT
                	JNZ	CRHD		;TRY AGAIN IF NOT TOO MANY ERRORS
                	JMP	$		;DYNAMIC ERROR HALT
                
                HDREAD	CALL	HDPREP		;PREPARE THE SECTOR HEADER IMAGE
                	RC			;ERROR EXIT
                	MVI	A,RSECT		;READ SECTOR COMMAND
                	OUT	HDCMND
                	CALL	PROCESS		;PROCESS THE READ
                	RC			;ERROR EXIT
                	XRA	A		;POINTER TO DATA BUFFER
                	OUT	HDCMND
                	MVI	B,SECLN/4	;NUMBER OF BYTES TO READ
                	LHLD	CDMAHD		;GET DESTINATION OF DATA
                	IN	HDDATA		;TWO DUMMY DATA BYTES
                	IN	HDDATA
                RTLOOP	IN	HDDATA		;MOVE FOUR BYTES
                	MOV	M,A		;BYTE ONE
                	INX	H
                	IN	HDDATA		;BYTE TWO
                	MOV	M,A
                	INX	H
                	IN	HDDATA		;BYTE THREE
                	MOV	M,A
                	INX	H
                	IN	HDDATA		;BYTE FOUR
                	MOV	M,A
                	INX	H
                	DCR	B		;UPDATE BYTE COUNT
                	JNZ	RTLOOP
                	RET
                
                PROCESS	IN	HDSTAT		;WAIT FOR COMMAND TO FINISH
                	MOV	B,A
                	ANI	OPDONE
                	JZ	PROCESS
                	MVI	A,DSKCLK	;TURN ON DISK CLOCK
                	OUT	HDCNTL
                	IN	HDSTAT
                	ANI	TMOUT		;TIMED OUT ?
                	STC
                	RNZ
                	IN	HDRESLT
                	ANI	RETRY		;ANY RETRIES ?
                	STC
                	RNZ
                	XRA	A		;NO ERROR EXIT
                	RET
                
                HDPREP	IN	HDSTAT		;IS DRIVE READY ?
                	ANI	DRVRDY
                	STC
                	RNZ
                	MVI	A,ISBUFF	;INITIALIZE POINTER TO HEADER BUFFER
                	OUT	HDCMND
                	MVI	A,NULL
                	OUT	HDFUNC		;SELECT DRIVE A
                	XRA	A
                	OUT	HDDATA		;FORM HEAD BYTE
                	OUT	HDDATA		;FORM TRACK BYTE
                	MVI	A,0		;FORM SECTOR BYTE
                HDSEC	EQU	$-1
                	OUT	HDDATA
                	MVI	A,80H		;FORM KEY
                	OUT	HDDATA
                	MVI	A,DSKCLK	;TURN ON DISK CLOCK
                	OUT	HDCNTL
                	MVI	A,WENABL	;WRITE ENABLE ON
                	OUT	HDCNTL
                	RET
                
                	ORG	BOOT+200H-2	;LAST WORD ON SECTOR IS LOAD ADDRESS
                
                CSTKHD	EQU	$
                	DW	BOOT
                	ENDIF
                
                	IF	MAXMW NE 0	;LOW LEVEL HDDMA ROUTINES
                CRDHD	CALL	HDSETUP		;SET UP PARAMETERS
                	LXI	B,RETRIES*100H+1 ;MAXIMUM # OF ATTEMPTS
                CRHD	PUSH	B		;SAVE ERROR COUNT
                	CALL	HDISSUE		;ATTEMPT THE READ
                	POP	B		;RESTORE THE ERROR COUNT
                	RNC			;RETURN IF NO ERROR
                	DCR	B		;UPDATE ERROR COUNT
                	JNZ	CRHD		;TRY AGAIN IF NOT TOO MANY ERRORS
                ERROR	JMP	$		;DYNAMIC ERROR HALT
                
                HDSETUP	SHLD	DMADMA		;SET UP DMA ADDRESS
                	CALL	HDRESET		;RESET CONTROLLER
                	LDA	HDSEC		;SET LOGICAL SECTOR NUMBER
                	DCR	A		;RANGE IS ACTAULLY 0-16
                	CALL	DIVSPT		;FIGURE OUT HEAD NUMBER -> (C)
                	ADI	SPT		;MAKE REAL SECTOR NUMBER
                	STA	DMARG3
                	MOV	A,C
                	STA	DMARG2		;SAVE HEAD NUMBER
                	CMA			;NEGATIVE LOGIC FOR THE CONTROLLER
                	ANI	7		;3 BITS OF HEAD SELECT
                	RLC			;SHOVE OVER TO BITS 2 - 4
                	RLC
                	STA	DMASEL1		;SAVE IN COMMAND CHANNEL HEAD SELECT
                	RET
                
                DIVSPT	MVI	C,0		;CLEAR HEAD COUNTER
                DIVSPTX	SUI	SPT		;SUBTRACT A TRACKS WORTH OF SECTORS
                	RC			;RETURN IF ALL DONE
                	INR	C		;BUMP TO NEXT HEAD
                	JMP	DIVSPTX
                
                HDRESET	MVI	A,(RET)		;ONE TIME CODE
                	STA	HDRESET
                	OUT	RESET		;SEND RESET PULSE TO CONTROLLER
                	LXI	H,DMACHAN	;ADDRESS OF COMMAND CHANNEL
                	SHLD	CHAN		;DEFAULT CHANNEL ADDRESS
                	XRA	A
                	STA	CHAN+2		;CLEAR EXTENDED ADDRESS BYTE
                	XTHL			;WAIT FOR RESET (AROUND 10 USEC'S)
                	XTHL
                	CALL	HDISSUE		;DO LOAD CONSTANTS
                	JC	ERROR		;CONTROLLER NOT PRESENT
                	MVI	A,DMASSTA	;SENSE STATUS COMMAND
                	STA	DMAOP
                RDYCHEK	CALL	HDISSUE
                	ANI	DREADY		;CHECK FOR DRIVE READY
                	JNZ	RDYCHEK		;LOOP IF NOT READY
                	LXI	H,0FFFFH
                	SHLD	DMASTEP		;DO RECALIBRATE
                	CALL	HDISSUE
                	LXI	H,0
                	SHLD	DMASTEP		;CLEAR STEP COUNTER
                	SHLD	DMARG0		;CLEAR CYLINDER #
                	SHLD	DMARG3		;CLEAR SECTOR # + READ DISK COMMAND
                	RET
                
                HDISSUE	LXI	H,DMASTAT	;STATUS BYTE
                	MVI	M,0
                	OUT	ATTN		;START CONTROLLER
                	LXI	D,0		;TIME OUT COUNTER
                	MOV	B,E		;CONTROLLER BUSY STATUS
                HDILOOP	MOV	A,M		;GET STATUS
                	ORA	A		;SET UP CPU FLAGS
                	RM			;RETURN NO ERROR (CARRY RESET)
                	STC
                	RNZ			;RETURN ERROR STATUS
                	XTHL			;WASTE SOME TIME
                	XTHL
                	XTHL
                	XTHL
                	DCX	D		;BUMP TIMEOUT COUNTER
                	MOV	A,D
                	ORA	E
                	JNZ	HDILOOP		;LOOP IF STILL BUSY
                	STC			;SET ERROR FLAG
                	RET
                
                HDSEC	DB	0		;CURRENTLY SELECTED SECTOR
                
                DMACHAN	EQU	$		;COMMAND CHANNEL AREA
                DMASEL0	DB	10H		;DRIVE SELECT (STEP OUT, DRIVE 0)
                DMASTEP	DW	0		;RELATIVE STEP COUNTER
                DMASEL1	DB	0		;HEAD SELECT
                DMADMA	DW	0		;DMA ADDRESS
                	DB	0		;EXTENDED ADDRESS
                DMARG0	DB	0		;FIRST ARGUMENT
                DMARG1	DB	STEPDLY		;SECOND ARGUMENT (STEPPING TIME)
                DMARG2	DB	HEADDLY		;THIRD ARGUMENT (SETTLE TIME)
                DMARG3	DB	SECTSIZ		;FOURTH ARGUMENT (SECTOR SIZE)
                DMAOP	DB	DMALCON		;OPERATION CODE
                DMASTAT	DB	0		;CONTROLLER STATUS BYTE
                DMALNK	DW	DMACHAN		;LINK ADDRESS TO NEXT COMMAND CHANNEL
                	DB	0		;EXTENDED ADDRESS
                
                	ORG	BOOT+200H
                CSTKHD	EQU	$		;STACK AREA AT END OF SECTOR
                	ENDIF	
                	ENDIF
                
                *****************************************************************
                *								*
                * COLD BOOT LOADER FOR THE DISK JOCKEY 2D REVISION B CONTROLLER	